@@ -73,13 +73,11 @@ void DragHelper::startDragInternal(QQuickItem *item, const QString &mimeType,
 {
     QPointer<QQuickItem> grabber = item;
 
-    QList<QUrl> urlList;
-    urlList.append(Backend::tryDecodeApplicationsUrl(url));
-
     QMimeData *dragData = new QMimeData();
+    const QByteArray &taskUrlData = Backend::tryDecodeApplicationsUrl(url).toString().toUtf8();
+    dragData->setData(QStringLiteral("text/x-orgkdeplasmataskmanager_taskurl"), taskUrlData);
     dragData->setData(mimeType, mimeData.toByteArray());
-    dragData->setData("application/x-orgkdeplasmataskmanager_taskbuttonitem", mimeData.toByteArray());
-    dragData->setUrls(urlList);
+    dragData->setData(QStringLiteral("application/x-orgkdeplasmataskmanager_taskbuttonitem"), mimeData.toByteArray());
 
     QDrag *drag = new QDrag(static_cast<QQuickItem *>(parent()));
     drag->setMimeData(dragData);
