@@ -1,5 +1,6 @@
 /*
  *  Copyright © 2003-2007 Fredrik Höglund <fredrik@kde.org>
+ *  Copyright © 2019 Benjamin Port <benjamin.port@enioka.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,8 +30,6 @@
 #include <KAboutData>
 #include <KPluginFactory>
 #include <KLocalizedString>
-#include <KConfig>
-#include <KConfigGroup>
 #include <KMessageBox>
 #include <KUrlRequesterDialog>
 #include <KIO/CopyJob>
@@ -40,14 +39,19 @@
 #include <KTar>
 #include <KGlobalSettings>
 
-#include <KNewStuff3/KNS3/DownloadDialog>
+#include <knewstuffcore_version.h>
+#if KNEWSTUFFCORE_VERSION_MAJOR==5 && KNEWSTUFFCORE_VERSION_MINOR>=68
+#include <KNSCore/EntryWrapper>
+#endif
 
+#include <QQmlListReference>
 #include <QX11Info>
 #include <QStandardItemModel>
 
 #include <X11/Xlib.h>
 #include <X11/Xcursor/Xcursor.h>
 
+#include "cursorthemesettings.h"
 #include <klauncher_iface.h>
 
 #ifdef HAVE_XFIXES
@@ -57,37 +61,35 @@
 K_PLUGIN_FACTORY_WITH_JSON(CursorThemeConfigFactory, "kcm_cursortheme.json", registerPlugin<CursorThemeConfig>();)
 
 CursorThemeConfig::CursorThemeConfig(QObject *parent, const QVariantList &args)
-    : KQuickAddons::ConfigModule(parent, args),
-      m_appliedSize(0),
-      m_preferredSize(0),
-      m_selectedThemeRow(-1),
-      m_selectedSizeRow(-1),
-      m_originalSelectedThemeRow(-1),
+    : KQuickAddons::ManagedConfigModule(parent, args),
+      m_settings(new CursorThemeSettings(this)),
       m_canInstall(true),
       m_canResize(true),
       m_canConfigure(true)
 {
+    m_preferredSize = m_settings->cursorSize();
+    connect(m_settings, &CursorThemeSettings::cursorThemeChanged, this, &CursorThemeConfig::updateSizeComboBox);
     qmlRegisterType<PreviewWidget>("org.kde.private.kcm_cursortheme", 1, 0, "PreviewWidget");
     qmlRegisterType<SortProxyModel>();
-
+    qmlRegisterType<CursorThemeSettings>();
     KAboutData* aboutData = new KAboutData(QStringLiteral("kcm_cursortheme"), i18n("Cursors"),
         QStringLiteral("1.0"), QString(), KAboutLicense::GPL, i18n("(c) 2003-2007 Fredrik Höglund"));
     aboutData->addAuthor(i18n("Fredrik Höglund"));
     aboutData->addAuthor(i18n("Marco Martin"));
     setAboutData(aboutData);
 
-    m_model = new CursorThemeModel(this);
+    m_themeModel = new CursorThemeModel(this);
 
-    m_proxyModel = new SortProxyModel(this);
-    m_proxyModel->setSourceModel(m_model);
-    m_proxyModel->setFilterCaseSensitivity(Qt::CaseSensitive);
-    m_proxyModel->sort(NameColumn, Qt::AscendingOrder);
+    m_themeProxyModel = new SortProxyModel(this);
+    m_themeProxyModel->setSourceModel(m_themeModel);
+    m_themeProxyModel->setFilterCaseSensitivity(Qt::CaseSensitive);
+    m_themeProxyModel->sort(NameColumn, Qt::AscendingOrder);
 
     m_sizesModel = new QStandardItemModel(this);
 
     // Disable the install button if we can't install new themes to ~/.icons,
     // or Xcursor isn't set up to look for cursor themes there.
-    if (!m_model->searchPaths().contains(QDir::homePath() + "/.icons") || !iconsIsWritable()) {
+    if (!m_themeModel->searchPaths().contains(QDir::homePath() + "/.icons") || !iconsIsWritable()) {
         setCanInstall(false);
     }
 }
@@ -97,6 +99,11 @@ CursorThemeConfig::~CursorThemeConfig()
     /* */
 }
 
+CursorThemeSettings *CursorThemeConfig::cursorThemeSettings() const
+{
+    return m_settings;
+}
+
 void CursorThemeConfig::setCanInstall(bool can)
 {
     if (m_canInstall == can) {
@@ -137,63 +144,33 @@ void CursorThemeConfig::setCanConfigure(bool can)
     emit canConfigureChanged();
 }
 
+int CursorThemeConfig::preferredSize() const
+{
+    return m_preferredSize;
+}
+
+void CursorThemeConfig::setPreferredSize(int size)
+{
+    if (m_preferredSize == size) {
+        return;
+    }
+    m_preferredSize = size;
+    emit preferredSizeChanged();
+}
+
 bool CursorThemeConfig::canConfigure() const
 {
     return m_canConfigure;
 }
 
-void CursorThemeConfig::setSelectedThemeRow(int row)
-{
-    if (m_selectedThemeRow == row) {
-        return;
-    }
-
-    m_selectedThemeRow = row;
-    emit selectedThemeRowChanged();
-    updateSizeComboBox();
-
-    QModelIndex selected = selectedIndex();
-
-    if (selected.isValid()) {
-        const CursorTheme *theme = m_proxyModel->theme(selected);
-    }
-    setNeedsSave(m_originalSelectedThemeRow != m_selectedThemeRow || m_originalPreferredSize != m_preferredSize);
-}
-
-int CursorThemeConfig::selectedThemeRow() const
-{
-    return m_selectedThemeRow;
-}
-
-void CursorThemeConfig::setSelectedSizeRow(int row)
-{
-    Q_ASSERT (row < m_sizesModel->rowCount() && row >= 0);
-
-    // we don't return early if m_selectedSizeRow == row as this is called after the model is changed
-    m_selectedSizeRow = row;
-    emit selectedSizeRowChanged();
-
-    int size = m_sizesModel->item(row)->data().toInt();
-
-    m_preferredSize = size;
-    setNeedsSave(m_originalSelectedThemeRow != m_selectedThemeRow || m_originalPreferredSize != m_preferredSize);
-}
-
-int CursorThemeConfig::selectedSizeRow() const
-{
-    return m_selectedSizeRow;
-}
-
 bool CursorThemeConfig::downloadingFile() const
 {
     return m_tempCopyJob;
 }
 
-
-
 QAbstractItemModel *CursorThemeConfig::cursorsModel()
 {
-    return m_proxyModel;
+    return m_themeProxyModel;
 }
 
 QAbstractItemModel *CursorThemeConfig::sizesModel()
@@ -213,16 +190,16 @@ bool CursorThemeConfig::iconsIsWritable() const
 
 void CursorThemeConfig::updateSizeComboBox()
 {
-
     // clear the combo box
     m_sizesModel->clear();
 
     // refill the combo box and adopt its icon size
-    QModelIndex selected = selectedIndex();
+    int row = cursorThemeIndex(m_settings->cursorTheme());
+    QModelIndex selected = m_themeProxyModel->index(row, 0);
     int maxIconWidth = 0;
     int maxIconHeight = 0;
     if (selected.isValid()) {
-        const CursorTheme *theme = m_proxyModel->theme(selected);
+        const CursorTheme *theme = m_themeProxyModel->theme(selected);
         const QList<int> sizes = theme->availableSizes();
         QIcon m_icon;
         // only refill the combobox if there is more that 1 size
@@ -256,47 +233,47 @@ void CursorThemeConfig::updateSizeComboBox()
                 item->setData(i);
                 m_sizesModel->appendRow(item);
                 comboBoxList << i;
-            };
+            }
 
             // select an item
-            int selectItem = comboBoxList.indexOf(m_preferredSize);
-            // m_preferredSize not available for this theme
+            int size = m_preferredSize;
+            int selectItem = comboBoxList.indexOf(size);
+
+            // cursor size not available for this theme
             if (selectItem < 0) {
-                /* Search the value next to m_preferredSize. The first entry (0)
-                   is ignored. (If m_preferredSize would have been 0, then we
-                   would had found it yet. As m_preferredSize is not 0, we won't
+                /* Search the value next to cursor size. The first entry (0)
+                   is ignored. (If cursor size would have been 0, then we
+                   would had found it yet. As cursor size is not 0, we won't
                    default to "automatic size".)*/
                 int j;
                 int distance;
                 int smallestDistance;
                 selectItem = 1;
                 j = comboBoxList.value(selectItem);
-                smallestDistance = j < m_preferredSize ? m_preferredSize - j : j - m_preferredSize;
+                size = j;
+                smallestDistance = qAbs(m_preferredSize - j);
                 for (int i = 2; i < comboBoxList.size(); ++i) {
                     j = comboBoxList.value(i);
-                    distance = j < m_preferredSize ? m_preferredSize - j : j - m_preferredSize;
+                    distance = qAbs(m_preferredSize - j);
                     if (distance < smallestDistance || (distance == smallestDistance && j > m_preferredSize)) {
                         smallestDistance = distance;
                         selectItem = i;
-                    };
+                        size = j;
+                    }
                 }
-            };
-            if (selectItem < 0) {
-                selectItem = 0;
             }
-            setSelectedSizeRow(selectItem);
-        };
-    };
+            m_settings->setCursorSize(size);
+        }
+    }
 
     // enable or disable the combobox
-    KConfig c("kcminputrc");
-    KConfigGroup cg(&c, "Mouse");
-    if (cg.isEntryImmutable("cursorSize")) {
+    if (m_settings->isImmutable("cursorSize")) {
         setCanResize(false);
     } else {
         setCanResize(m_sizesModel->rowCount() > 0);
-    };
-
+    }
+    // We need to emit a cursorSizeChanged in all case to refresh UI
+    emit m_settings->cursorSizeChanged();
 }
 
 bool CursorThemeConfig::applyTheme(const CursorTheme *theme, const int size)
@@ -324,9 +301,6 @@ bool CursorThemeConfig::applyTheme(const CursorTheme *theme, const int size)
     // Update the Xcursor X resources
     runRdb(0);
 
-    // Notify all applications that the cursor theme has changed
-    KGlobalSettings::self()->emitChange(KGlobalSettings::CursorChanged);
-
     // Reload the standard cursors
     QStringList names;
 
@@ -350,7 +324,8 @@ bool CursorThemeConfig::applyTheme(const CursorTheme *theme, const int size)
     foreach (const QString &name, names) {
         XFixesChangeCursorByName(QX11Info::display(), theme->loadCursor(name, size), QFile::encodeName(name));
     }
-
+    updateSizeComboBox();
+    emit themeApplied();
     return true;
 #else
     Q_UNUSED(theme)
@@ -358,140 +333,117 @@ bool CursorThemeConfig::applyTheme(const CursorTheme *theme, const int size)
 #endif
 }
 
+int CursorThemeConfig::cursorSizeIndex(int cursorSize) const
+{
+    if (m_sizesModel->rowCount() > 0) {
+         if (cursorSize  == 0) {
+             return 0;
+         }
+         const auto items = m_sizesModel->findItems(QString::number(cursorSize));
+         if (items.count() == 1) {
+             return items.first()->row();
+         }
+    }
+    return -1;
+}
+
+int CursorThemeConfig::cursorSizeFromIndex(int index)
+{
+    Q_ASSERT (index < m_sizesModel->rowCount() && index >= 0);
+
+    return m_sizesModel->item(index)->data().toInt();
+}
+
+int CursorThemeConfig::cursorThemeIndex(const QString &cursorTheme) const
+{
+    auto results = m_themeProxyModel->findIndex(cursorTheme);
+    return results.row();
+}
+
+QString CursorThemeConfig::cursorThemeFromIndex(int index) const
+{
+    QModelIndex idx = m_themeProxyModel->index(index, 0);
+    return m_themeProxyModel->theme(idx)->name();
+}
 
 void CursorThemeConfig::save()
 {
-    const CursorTheme *theme = selectedIndex().isValid() ? m_proxyModel->theme(selectedIndex()) : nullptr;
+    ManagedConfigModule::save();
+    setPreferredSize(m_settings->cursorSize());
 
-    KConfig config("kcminputrc");
-    KConfigGroup c(&config, "Mouse");
-    if (theme) {
-        c.writeEntry("cursorTheme", theme->name());
-    };
-    c.writeEntry("cursorSize", m_preferredSize);
-    c.sync();
+    int row = cursorThemeIndex(m_settings->cursorTheme());
+    QModelIndex selected = m_themeProxyModel->index(row, 0);
+    const CursorTheme *theme = selected.isValid() ? m_themeProxyModel->theme(selected) : nullptr;
 
-    if (!applyTheme(theme, m_preferredSize)) {
+    if (!applyTheme(theme, m_settings->cursorSize())) {
         emit showInfoMessage(i18n("You have to restart the Plasma session for these changes to take effect."));
     }
 
-    m_appliedIndex = selectedIndex();
-    m_appliedSize = m_preferredSize;
-    m_originalSelectedThemeRow = m_selectedThemeRow;
-    m_originalPreferredSize = m_preferredSize;
-    setNeedsSave(false);
+    KGlobalSettings::self()->emitChange(KGlobalSettings::CursorChanged);
 }
 
 
 void CursorThemeConfig::load()
 {
-
-    // Get the name of the theme libXcursor currently uses
-    QString currentTheme;
-    if (QX11Info::isPlatformX11()) {
-        currentTheme = XcursorGetTheme(QX11Info::display());
-    }
-
+    ManagedConfigModule::load();
+    setPreferredSize(m_settings->cursorSize());
     // Get the name of the theme KDE is configured to use
-    KConfig c("kcminputrc");
-    KConfigGroup cg(&c, "Mouse");
-    currentTheme = cg.readEntry("cursorTheme", currentTheme);
-
-    // Find the theme in the listview
-    if (!currentTheme.isEmpty()) {
-        m_appliedIndex = m_proxyModel->findIndex(currentTheme);
-    } else {
-        m_appliedIndex = m_proxyModel->defaultIndex();
-    }
+    QString currentTheme = m_settings->cursorTheme();
 
     // Disable the listview and the buttons if we're in kiosk mode
-    if (cg.isEntryImmutable("cursorTheme")) {
+    if (m_settings->isImmutable( QStringLiteral( "cursorTheme" ))) {
           setCanConfigure(false);
           setCanInstall(false);
     }
 
-    const CursorTheme *theme = m_proxyModel->theme(m_appliedIndex);
-
-    setSelectedThemeRow(m_appliedIndex.row());
-    m_originalSelectedThemeRow = m_selectedThemeRow;
-
-    // Load cursor size
-    int size = cg.readEntry("cursorSize", 0);
-    if (size <= 0) {
-        m_preferredSize = 0;
-    } else {
-        m_preferredSize = size;
-    }
-    m_originalPreferredSize = m_preferredSize;
     updateSizeComboBox(); // This handles also the kiosk mode
 
-    m_appliedSize = size;
-
-
     setNeedsSave(false);
 }
 
 
 void CursorThemeConfig::defaults()
 {
-    QModelIndex defaultIndex = m_proxyModel->findIndex("breeze_cursors");
-    setSelectedThemeRow(defaultIndex.row());
-    m_preferredSize = 0;
-    updateSizeComboBox();
-    setNeedsSave(m_originalSelectedThemeRow != m_selectedThemeRow || m_originalPreferredSize != m_preferredSize);
+    ManagedConfigModule::defaults();
+    m_preferredSize = m_settings->cursorSize();
 }
 
-
-void CursorThemeConfig::selectionChanged()
+void CursorThemeConfig::ghnsEntriesChanged(const QQmlListReference &changedEntries)
 {
-    updateSizeComboBox();
-
-    setNeedsSave(m_originalSelectedThemeRow != m_selectedThemeRow || m_originalPreferredSize != m_preferredSize);
-    //setNeedsSave(m_appliedIndex != selectedIndex());
-}
-
-QModelIndex CursorThemeConfig::selectedIndex() const
-{
-    return m_proxyModel->index(m_selectedThemeRow, 0);
-}
-
-void CursorThemeConfig::getNewClicked()
-{
-    KNS3::DownloadDialog dialog("xcursor.knsrc", nullptr);
-    if (dialog.exec()) {
-        KNS3::Entry::List list = dialog.changedEntries();
-        if (!list.isEmpty()) {
-            for (const KNS3::Entry& entry : list) {
-                if (entry.status() == KNS3::Entry::Deleted) {
-                    for (const QString& deleted : entry.uninstalledFiles()) {
-                        QVector<QStringRef> list = deleted.splitRef(QLatin1Char('/'));
-                        if (list.last() == QLatin1Char('*')) {
-                            list.takeLast();
-                        }
-                        QModelIndex idx = m_model->findIndex(list.last().toString());
-                        if (idx.isValid()) {
-                            m_model->removeTheme(idx);
-                        }
+#if KNEWSTUFFCORE_VERSION_MAJOR==5 && KNEWSTUFFCORE_VERSION_MINOR>=68
+    for (int i = 0; i < changedEntries.count(); ++i) {
+        KNSCore::EntryWrapper* entry = qobject_cast<KNSCore::EntryWrapper*>(changedEntries.at(i));
+        if (entry) {
+            if (entry->entry().status() == KNS3::Entry::Deleted) {
+                for (const QString& deleted : entry->entry().uninstalledFiles()) {
+                    QVector<QStringRef> list = deleted.splitRef(QLatin1Char('/'));
+                    if (list.last() == QLatin1Char('*')) {
+                        list.takeLast();
                     }
-                } else if (entry.status() == KNS3::Entry::Installed) {
-                    for (const QString& created : entry.installedFiles()) {
-                        QStringList list = created.split(QLatin1Char('/'));
-                        if (list.last() == QLatin1Char('*')) {
-                            list.takeLast();
-                        }
-                        // Because we sometimes get some extra slashes in the installed files list
-                        list.removeAll({});
-                        // Because we'll also get the containing folder, if it was not already there
-                        // we need to ignore it.
-                        if (list.last() == QLatin1String(".icons")) {
-                            continue;
-                        }
-                        m_model->addTheme(list.join(QLatin1Char('/')));
+                    QModelIndex idx = m_themeModel->findIndex(list.last().toString());
+                    if (idx.isValid()) {
+                        m_themeModel->removeTheme(idx);
                     }
                 }
+            } else if (entry->entry().status() == KNS3::Entry::Installed) {
+                for (const QString& created : entry->entry().installedFiles()) {
+                    QStringList list = created.split(QLatin1Char('/'));
+                    if (list.last() == QLatin1Char('*')) {
+                        list.takeLast();
+                    }
+                    // Because we sometimes get some extra slashes in the installed files list
+                    list.removeAll({});
+                    // Because we'll also get the containing folder, if it was not already there
+                    // we need to ignore it.
+                    if (list.last() == QLatin1String(".icons")) {
+                        continue;
+                    }
+                    m_themeModel->addTheme(list.join(QLatin1Char('/')));
+                }
             }
         }
     }
+#endif
 }
 
 void CursorThemeConfig::installThemeFromFile(const QUrl &url)
@@ -559,7 +511,7 @@ void CursorThemeConfig::installThemeFile(const QString &path)
     if (!QDir().mkpath(destDir)) {
         emit showErrorMessage(i18n("Failed to create 'icons' folder."));
         return;
-    };
+    }
 
     // Process each cursor theme in the archive
     foreach (const QString &dirName, themeDirs) {
@@ -590,27 +542,27 @@ void CursorThemeConfig::installThemeFile(const QString &path)
         const KArchiveDirectory *dir = static_cast<const KArchiveDirectory*>
                         (archiveDir->entry(dirName));
         dir->copyTo(dest.path());
-        m_model->addTheme(dest);
+        m_themeModel->addTheme(dest);
     }
 
     archive.close();
 
     emit showSuccessMessage(i18n("Theme installed successfully."));
 
-    m_model->refreshList();
+    m_themeModel->refreshList();
 }
 
 void CursorThemeConfig::removeTheme(int row)
 {
-    QModelIndex idx = m_proxyModel->index(row, 0);
+    QModelIndex idx = m_themeProxyModel->index(row, 0);
     if (!idx.isValid()) {
         return;
     }
 
-    const CursorTheme *theme = m_proxyModel->theme(idx);
+    const CursorTheme *theme = m_themeProxyModel->theme(idx);
 
     // Don't let the user delete the currently configured theme
-    if (idx == m_appliedIndex) {
+    if (theme->name() == m_settings->cursorTheme()) {
         KMessageBox::sorry(nullptr, i18n("<qt>You cannot delete the theme you are currently "
                 "using.<br />You have to switch to another theme first.</qt>"));
         return;
@@ -633,7 +585,7 @@ void CursorThemeConfig::removeTheme(int row)
     KIO::del(QUrl::fromLocalFile(theme->path())); // async
 
     // Remove the theme from the model
-    m_proxyModel->removeTheme(idx);
+    m_themeProxyModel->removeTheme(idx);
 
     // TODO:
     //  Since it's possible to substitute cursors in a system theme by adding a local
