@@ -55,13 +55,13 @@ class CFontViewPart : public KParts::ReadOnlyPart
     public:
 
     CFontViewPart(QWidget *parentWidget, QObject *parent, const QList<QVariant> &args);
-    virtual ~CFontViewPart();
+    ~CFontViewPart() override;
 
-    bool openUrl(const QUrl &url) Q_DECL_OVERRIDE;
+    bool openUrl(const QUrl &url) override;
 
     protected:
 
-    bool openFile() Q_DECL_OVERRIDE;
+    bool openFile() override;
 
     public Q_SLOTS:
 
