@@ -5,6 +5,8 @@
  *  based on kcmtaskbar.cpp
  *  Copyright (c) 2000 Kurt Granroth <granroth@kde.org>
  *
+ *  Copyright (c) 2019 Kevin Ottens <kevin.ottens@enioka.com>
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -28,9 +30,6 @@
 
 #include <QVBoxLayout>
 
-#include <kconfig.h>
-#include <kconfiggroup.h>
-#include <ksharedconfig.h>
 #include <kworkspace.h>
 #include <qregexp.h>
 #include <kdesktopfile.h>
@@ -41,12 +40,13 @@
 #include <QLineEdit>
 
 #include "kcmsmserver.h"
-#include "smserverconfigimpl.h"
+#include "smserversettings.h"
+#include "ui_smserverconfigdlg.h"
+
 #include <KPluginFactory>
 #include <KPluginLoader>
 #include <KLocalizedString>
 
-#include "kworkspace.h"
 #include <sessionmanagement.h>
 
 #include "login1_manager.h"
@@ -55,11 +55,15 @@ K_PLUGIN_FACTORY(SMSFactory, registerPlugin<SMServerConfig>();)
 
 SMServerConfig::SMServerConfig(QWidget *parent, const QVariantList &args)
   : KCModule(parent, args)
+  , ui(new Ui::SMServerConfigDlg)
+  , m_settings(new SMServerSettings(this))
   , m_login1Manager(new OrgFreedesktopLogin1ManagerInterface(QStringLiteral("org.freedesktop.login1"),
                                                              QStringLiteral("/org/freedesktop/login1"),
                                                              QDBusConnection::systemBus(),
                                                              this))
 {
+    ui->setupUi(this);
+
     setQuickHelp( i18n("<h1>Session Manager</h1>"
     " You can configure the session manager here."
     " This includes options such as whether or not the session exit (logout)"
@@ -67,17 +71,17 @@ SMServerConfig::SMServerConfig(QWidget *parent, const QVariantList &args)
     " and whether the computer should be automatically shut down after session"
     " exit by default."));
 
-    QVBoxLayout *topLayout = new QVBoxLayout(this);
-    topLayout->setContentsMargins(0, 0, 0, 0);
-    dialog = new SMServerConfigImpl(this);
-    connect(dialog, SIGNAL(changed()), SLOT(changed()));
-
     initFirmwareSetup();
     checkFirmwareSetupRequested();
 
-    topLayout->addWidget(dialog);
+    ui->firmwareSetupBox->hide();
+    ui->firmwareSetupMessageWidget->hide();
+
+    addConfig(m_settings, this);
 }
 
+SMServerConfig::~SMServerConfig() = default;
+
 void SMServerConfig::initFirmwareSetup()
 {
     m_rebootNowAction = new QAction(QIcon::fromTheme(QStringLiteral("system-reboot")), i18n("Restart Now"));
@@ -94,9 +98,9 @@ void SMServerConfig::initFirmwareSetup()
         }
     });
 
-    connect(dialog->firmwareSetupCheck, &QCheckBox::clicked, this, [this](bool enable) {
-        dialog->firmwareSetupMessageWidget->removeAction(m_rebootNowAction);
-        dialog->firmwareSetupMessageWidget->animatedHide();
+    connect(ui->firmwareSetupCheck, &QCheckBox::clicked, this, [this](bool enable) {
+        ui->firmwareSetupMessageWidget->removeAction(m_rebootNowAction);
+        ui->firmwareSetupMessageWidget->animatedHide();
 
         QDBusMessage message = QDBusMessage::createMethodCall(m_login1Manager->service(),
                                                               m_login1Manager->path(),
@@ -115,7 +119,7 @@ void SMServerConfig::initFirmwareSetup()
 
             checkFirmwareSetupRequested();
 
-            KMessageWidget *message = dialog->firmwareSetupMessageWidget;
+            KMessageWidget *message = ui->firmwareSetupMessageWidget;
 
             if (reply.isError()) {
                 // User likely canceled the PolKit prompt, don't show an error in this case
@@ -147,88 +151,17 @@ void SMServerConfig::initFirmwareSetup()
         // now check whether we're UEFI to provide a more descriptive button label
         if (QFileInfo(QStringLiteral("/sys/firmware/efi")).isDir()) {
             m_isUefi = true;
-            dialog->firmwareSetupBox->setTitle(i18n("UEFI Setup"));
-            dialog->firmwareSetupCheck->setText(i18n("Enter UEFI setup on next restart"));
+            ui->firmwareSetupBox->setTitle(i18n("UEFI Setup"));
+            ui->firmwareSetupCheck->setText(i18n("Enter UEFI setup on next restart"));
         }
 
-        dialog->firmwareSetupBox->setVisible(true);
+        ui->firmwareSetupBox->setVisible(true);
     }
 }
 
 void SMServerConfig::checkFirmwareSetupRequested()
 {
-    dialog->firmwareSetupCheck->setChecked(m_login1Manager->property("RebootToFirmwareSetup").toBool());
-}
-
-void SMServerConfig::load()
-{
-  KConfigGroup c(KSharedConfig::openConfig(QStringLiteral("ksmserverrc"), KConfig::NoGlobals),
-                 QStringLiteral("General"));
-  dialog->confirmLogoutCheck->setChecked(c.readEntry("confirmLogout", true));
-  bool en = c.readEntry("offerShutdown", true);
-  dialog->offerShutdownCheck->setChecked(en);
-  dialog->sdGroup->setEnabled(en);
-
-  QString s = c.readEntry( "loginMode" );
-  if ( s == QLatin1String("default") )
-      dialog->emptySessionRadio->setChecked(true);
-  else if ( s == QLatin1String("restoreSavedSession") )
-      dialog->savedSessionRadio->setChecked(true);
-  else // "restorePreviousLogout"
-      dialog->previousSessionRadio->setChecked(true);
-
-  switch (c.readEntry("shutdownType", int(KWorkSpace::ShutdownTypeNone))) {
-  case int(KWorkSpace::ShutdownTypeHalt):
-    dialog->haltRadio->setChecked(true);
-    break;
-  case int(KWorkSpace::ShutdownTypeReboot):
-    dialog->rebootRadio->setChecked(true);
-    break;
-  default:
-    dialog->logoutRadio->setChecked(true);
-    break;
-  }
-  dialog->excludeLineedit->setText( c.readEntry("excludeApps"));
-
-  emit changed(false);
-}
-
-void SMServerConfig::save()
-{
-  KConfig c(QStringLiteral("ksmserverrc"), KConfig::NoGlobals);
-  KConfigGroup group = c.group(QStringLiteral("General"));
-  group.writeEntry( "confirmLogout", dialog->confirmLogoutCheck->isChecked());
-  group.writeEntry( "offerShutdown", dialog->offerShutdownCheck->isChecked());
-  QString s = QStringLiteral("restorePreviousLogout");
-  if ( dialog->emptySessionRadio->isChecked() )
-      s = QStringLiteral("default");
-  else if ( dialog->savedSessionRadio->isChecked() )
-      s = QStringLiteral("restoreSavedSession");
-  group.writeEntry( "loginMode", s );
-
-  group.writeEntry( "shutdownType",
-                 dialog->haltRadio->isChecked() ?
-                   int(KWorkSpace::ShutdownTypeHalt) :
-                   dialog->rebootRadio->isChecked() ?
-                     int(KWorkSpace::ShutdownTypeReboot) :
-                     int(KWorkSpace::ShutdownTypeNone));
-  group.writeEntry("excludeApps", dialog->excludeLineedit->text());
-  c.sync();
-# if 0
-  // update the k menu if necessary
-  QDBusInterface kicker("org.kde.kicker", "/kicker", "org.kde.kicker");
-  kicker.call("configure");
-#endif
-}
-
-void SMServerConfig::defaults()
-{
-  dialog->previousSessionRadio->setChecked(true);
-  dialog->confirmLogoutCheck->setChecked(true);
-  dialog->offerShutdownCheck->setChecked(true);
-  dialog->sdGroup->setEnabled(true);
-  dialog->logoutRadio->setChecked(true);
-  dialog->excludeLineedit->clear();
+    ui->firmwareSetupCheck->setChecked(m_login1Manager->property("RebootToFirmwareSetup").toBool());
 }
 
 #include "kcmsmserver.moc"
