@@ -1,6 +1,7 @@
 /* This file is part of the KDE Project
    Copyright (c) 2014 Marco Martin <mart@kde.org>
    Copyright (c) 2014 Vishesh Handa <me@vhanda.in>
+   Copyright (c) 2019 Cyril Rossi <cyril.rossi@enioka.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,42 +20,45 @@
 
 #include "kcm.h"
 #include "../krdb/krdb.h"
-#include "../cursortheme/xcursor/xcursortheme.h"
 #include "config-kcm.h"
 #include "config-workspace.h"
 #include <klauncher_iface.h>
 
 #include <KAboutData>
 #include <KSharedConfig>
-#include <QDebug>
-#include <QStandardPaths>
-#include <QProcess>
-#include <QQuickWidget>
-#include <QQuickView>
 #include <KGlobalSettings>
 #include <KIconLoader>
 #include <KAutostart>
 #include <KRun>
+#include <KService>
 
-#include <QQmlEngine>
-#include <QQmlContext>
+#include <QDebug>
 #include <QQuickItem>
 #include <QQuickWindow>
+#include <QStandardPaths>
+#include <QProcess>
 #include <QStandardItemModel>
 #include <QX11Info>
 
 #include <KLocalizedString>
-#include <Plasma/PluginLoader>
+#include <KPackage/PackageLoader>
 
 #include <X11/Xlib.h>
-#include <X11/Xcursor/Xcursor.h>
+
+#include "lookandfeelsettings.h"
+
+#ifdef HAVE_XCURSOR
+#   include "../cursortheme/xcursor/xcursortheme.h"
+#   include <X11/Xcursor/Xcursor.h>
+#endif
 
 #ifdef HAVE_XFIXES
 #  include <X11/extensions/Xfixes.h>
 #endif
 
-KCMLookandFeel::KCMLookandFeel(QObject* parent, const QVariantList& args)
-    : KQuickAddons::ConfigModule(parent, args)
+KCMLookandFeel::KCMLookandFeel(QObject *parent, const QVariantList &args)
+    : KQuickAddons::ManagedConfigModule(parent, args)
+    , m_settings(new LookAndFeelSettings(this))
     , m_config(QStringLiteral("kdeglobals"))
     , m_configGroup(m_config.group("KDE"))
     , m_applyColors(true)
@@ -67,9 +71,11 @@ KCMLookandFeel::KCMLookandFeel(QObject* parent, const QVariantList& args)
     , m_resetDefaultLayout(false)
     , m_applyWindowDecoration(true)
 {
+    qmlRegisterType<LookAndFeelSettings>();
     qmlRegisterType<QStandardItemModel>();
     qmlRegisterType<KCMLookandFeel>();
-    KAboutData* about = new KAboutData(QStringLiteral("kcm_lookandfeel"), i18n("Global Theme"),
+
+    KAboutData *about = new KAboutData(QStringLiteral("kcm_lookandfeel"), i18n("Global Theme"),
                                        QStringLiteral("0.1"), QString(), KAboutLicense::LGPL);
     about->addAuthor(i18n("Marco Martin"), QString(), QStringLiteral("mart@kde.org"));
     setAboutData(about);
@@ -79,7 +85,7 @@ KCMLookandFeel::KCMLookandFeel(QObject* parent, const QVariantList& args)
     QHash<int, QByteArray> roles = m_model->roleNames();
     roles[PluginNameRole] = "pluginName";
     roles[DescriptionRole] = "description";
-    roles[ScreenhotRole] = "screenshot";
+    roles[ScreenshotRole] = "screenshot";
     roles[FullScreenPreviewRole] = "fullScreenPreview";
     roles[HasSplashRole] = "hasSplash";
     roles[HasLockScreenRole] = "hasLockScreen";
@@ -101,62 +107,29 @@ KCMLookandFeel::~KCMLookandFeel()
 {
 }
 
-void KCMLookandFeel::getNewStuff(QQuickItem *ctx)
+void KCMLookandFeel::reloadModel()
 {
-    if (!m_newStuffDialog) {
-        m_newStuffDialog = new KNS3::DownloadDialog( QLatin1String("lookandfeel.knsrc") );
-        m_newStuffDialog.data()->setWindowTitle(i18n("Download New Global Themes"));
-        m_newStuffDialog->setWindowModality(Qt::WindowModal);
-        m_newStuffDialog->winId(); // so it creates the windowHandle();
-        connect(m_newStuffDialog.data(), &KNS3::DownloadDialog::accepted, this,  &KCMLookandFeel::loadModel);
-    }
-
-    if (ctx && ctx->window()) {
-        m_newStuffDialog->windowHandle()->setTransientParent(ctx->window());
-    }
-
-    m_newStuffDialog.data()->show();
+    loadModel();
 }
 
-QStandardItemModel *KCMLookandFeel::lookAndFeelModel()
+QStandardItemModel *KCMLookandFeel::lookAndFeelModel() const
 {
     return m_model;
 }
 
-QString KCMLookandFeel::selectedPlugin() const
+int KCMLookandFeel::pluginIndex(const QString &pluginName) const
 {
-    return m_selectedPlugin;
-}
-
-void KCMLookandFeel::setSelectedPlugin(const QString &plugin)
-{
-    if (m_selectedPlugin == plugin) {
-        return;
+    const auto results = m_model->match(m_model->index(0, 0), PluginNameRole, pluginName);
+    if (results.count() == 1) {
+        return results.first().row();
     }
 
-    const bool firstTime = m_selectedPlugin.isNull();
-    m_selectedPlugin = plugin;
-    emit selectedPluginChanged();
-    emit selectedPluginIndexChanged();
-
-    if (!firstTime) {
-        setNeedsSave(true);
-    }
-}
-
-int KCMLookandFeel::selectedPluginIndex() const
-{
-    for (int i = 0; i < m_model->rowCount(); ++i) {
-        if (m_model->data(m_model->index(i, 0), PluginNameRole).toString() == m_selectedPlugin) {
-            return i;
-        }
-    }
     return -1;
 }
 
-QList<Plasma::Package> KCMLookandFeel::availablePackages(const QStringList &components)
+QList<KPackage::Package> KCMLookandFeel::availablePackages(const QStringList &components)
 {
-    QList<Plasma::Package> packages;
+    QList<KPackage::Package> packages;
     QStringList paths;
     const QStringList dataPaths = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
 
@@ -167,9 +140,9 @@ QList<Plasma::Package> KCMLookandFeel::availablePackages(const QStringList &comp
     }
 
     for (const QString &path : paths) {
-        Plasma::Package pkg = Plasma::PluginLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
+        KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
         pkg.setPath(path);
-        pkg.setFallbackPackage(Plasma::Package());
+        pkg.setFallbackPackage(KPackage::Package());
         if (components.isEmpty()) {
             packages << pkg;
         } else {
@@ -185,19 +158,24 @@ QList<Plasma::Package> KCMLookandFeel::availablePackages(const QStringList &comp
     return packages;
 }
 
+LookAndFeelSettings *KCMLookandFeel::lookAndFeelSettings() const
+{
+    return m_settings;
+}
+
 void KCMLookandFeel::loadModel()
 {
     m_model->clear();
 
-    const QList<Plasma::Package> pkgs = availablePackages({"defaults", "layouts"});
-    for (const Plasma::Package &pkg : pkgs) {
+    const QList<KPackage::Package> pkgs = availablePackages({"defaults", "layouts"});
+    for (const KPackage::Package &pkg : pkgs) {
         if (!pkg.metadata().isValid()) {
             continue;
         }
-        QStandardItem* row = new QStandardItem(pkg.metadata().name());
-        row->setData(pkg.metadata().pluginName(), PluginNameRole);
-        row->setData(pkg.metadata().comment(), DescriptionRole);
-        row->setData(pkg.filePath("preview"), ScreenhotRole);
+        QStandardItem *row = new QStandardItem(pkg.metadata().name());
+        row->setData(pkg.metadata().pluginId(), PluginNameRole);
+        row->setData(pkg.metadata().description(), DescriptionRole);
+        row->setData(pkg.filePath("preview"), ScreenshotRole);
         row->setData(pkg.filePath("fullscreenpreview"), FullScreenPreviewRole);
 
         //What the package provides
@@ -241,45 +219,36 @@ void KCMLookandFeel::loadModel()
         m_model->appendRow(row);
     }
     m_model->sort(0 /*column*/);
-    emit selectedPluginIndexChanged();
+
+    //Model has been cleared so pretend the selected look and fell changed to force view update
+    emit m_settings->lookAndFeelPackageChanged();
 }
 
 void KCMLookandFeel::load()
 {
-    m_package = Plasma::PluginLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
-    KConfigGroup cg(KSharedConfig::openConfig(QStringLiteral("kdeglobals")), "KDE");
-    const QString packageName = cg.readEntry("LookAndFeelPackage", QString());
-    if (!packageName.isEmpty()) {
-        m_package.setPath(packageName);
-    }
+    ManagedConfigModule::load();
 
-    if (!m_package.metadata().isValid()) {
-        return;
-    }
-
-    setSelectedPlugin(m_package.metadata().pluginName());
-
-    setNeedsSave(false);
+    m_package = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
+    m_package.setPath(m_settings->lookAndFeelPackage());
 }
 
-
 void KCMLookandFeel::save()
 {
-    Plasma::Package package = Plasma::PluginLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
-    package.setPath(m_selectedPlugin);
+    KPackage::Package package = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("Plasma/LookAndFeel"));
+    package.setPath(m_settings->lookAndFeelPackage());
 
     if (!package.isValid()) {
         return;
     }
 
-    m_configGroup.writeEntry("LookAndFeelPackage", m_selectedPlugin);
+    ManagedConfigModule::save();
 
     if (m_resetDefaultLayout) {
         QDBusMessage message = QDBusMessage::createMethodCall(QStringLiteral("org.kde.plasmashell"), QStringLiteral("/PlasmaShell"),
                                                    QStringLiteral("org.kde.PlasmaShell"), QStringLiteral("loadLookAndFeelDefaultLayout"));
 
         QList<QVariant> args;
-        args << m_selectedPlugin;
+        args << m_settings->lookAndFeelPackage();
         message.setArguments(args);
 
         QDBusConnection::sessionBus().call(message, QDBus::NoBlock);
@@ -422,23 +391,14 @@ void KCMLookandFeel::save()
     }
 
     //TODO: option to enable/disable apply? they don't seem required by UI design
-    setSplashScreen(m_selectedPlugin);
-    setLockScreen(m_selectedPlugin);
+    setSplashScreen(m_settings->lookAndFeelPackage());
+    setLockScreen(m_settings->lookAndFeelPackage());
 
     m_configGroup.sync();
-    m_package.setPath(m_selectedPlugin);
+    m_package.setPath(m_settings->lookAndFeelPackage());
     runRdb(KRdbExportQtColors | KRdbExportGtkTheme | KRdbExportColors | KRdbExportQtSettings | KRdbExportXftSettings);
 }
 
-void KCMLookandFeel::defaults()
-{
-    if (!m_package.metadata().isValid()) {
-        return;
-    }
-
-    setSelectedPlugin(m_package.metadata().pluginName());
-}
-
 void KCMLookandFeel::setWidgetStyle(const QString &style)
 {
     if (style.isEmpty()) {
@@ -510,10 +470,11 @@ void KCMLookandFeel::setCursorTheme(const QString themeName)
     cg.writeEntry("cursorTheme", themeName);
     cg.sync();
 
+#ifdef HAVE_XCURSOR
     // Require the Xcursor version that shipped with X11R6.9 or greater, since
     // in previous versions the Xfixes code wasn't enabled due to a bug in the
     // build system (freedesktop bug #975).
-#if HAVE_XFIXES && XFIXES_MAJOR >= 2 && XCURSOR_LIB_VERSION >= 10105
+#if defined(HAVE_XFIXES) && XFIXES_MAJOR >= 2 && XCURSOR_LIB_VERSION >= 10105
     const int cursorSize = cg.readEntry("cursorSize", 0);
 
     QDir themeDir = cursorThemeDir(themeName, 0);
@@ -569,6 +530,7 @@ void KCMLookandFeel::setCursorTheme(const QString themeName)
                                  i18n("You have to restart the Plasma session for these changes to take effect."),
                                  i18n("Cursor Settings Changed"), "CursorSettingsChanged");
 #endif
+#endif
 }
 
 QDir KCMLookandFeel::cursorThemeDir(const QString &theme, const int depth)
@@ -622,6 +584,7 @@ const QStringList KCMLookandFeel::cursorSearchPaths()
     if (!m_cursorSearchPaths.isEmpty())
         return m_cursorSearchPaths;
 
+#ifdef HAVE_XCURSOR
 #if XCURSOR_LIB_MAJOR == 1 && XCURSOR_LIB_MINOR < 1
     // These are the default paths Xcursor will scan for cursor themes
     QString path("~/.icons:/usr/share/icons:/usr/share/pixmaps:/usr/X11R6/lib/X11/icons");
@@ -652,6 +615,9 @@ const QStringList KCMLookandFeel::cursorSearchPaths()
     // Expand all occurrences of ~/ to the home dir
     m_cursorSearchPaths.replaceInStrings(QRegExp(QStringLiteral("^~\\/")), QDir::home().path() + QLatin1Char('/'));
     return m_cursorSearchPaths;
+#else
+    return QStringList();
+#endif
 }
 
 void KCMLookandFeel::setSplashScreen(const QString &theme)
@@ -724,94 +690,6 @@ void KCMLookandFeel::setWindowDecoration(const QString &library, const QString &
     QDBusConnection::sessionBus().send(message);
 }
 
-void KCMLookandFeel::setApplyColors(bool apply)
-{
-    if (m_applyColors == apply) {
-        return;
-    }
-
-    m_applyColors = apply;
-    emit applyColorsChanged();
-}
-
-bool KCMLookandFeel::applyColors() const
-{
-    return m_applyColors;
-}
-
-void KCMLookandFeel::setApplyWidgetStyle(bool apply)
-{
-    if (m_applyWidgetStyle == apply) {
-        return;
-    }
-
-    m_applyWidgetStyle = apply;
-    emit applyWidgetStyleChanged();
-}
-
-bool KCMLookandFeel::applyWidgetStyle() const
-{
-    return m_applyWidgetStyle;
-}
-
-void KCMLookandFeel::setApplyIcons(bool apply)
-{
-    if (m_applyIcons == apply) {
-        return;
-    }
-
-    m_applyIcons = apply;
-    emit applyIconsChanged();
-}
-
-bool KCMLookandFeel::applyIcons() const
-{
-    return m_applyIcons;
-}
-
-void KCMLookandFeel::setApplyPlasmaTheme(bool apply)
-{
-    if (m_applyPlasmaTheme == apply) {
-        return;
-    }
-
-    m_applyPlasmaTheme = apply;
-    emit applyPlasmaThemeChanged();
-}
-
-bool KCMLookandFeel::applyPlasmaTheme() const
-{
-    return m_applyPlasmaTheme;
-}
-
-void KCMLookandFeel::setApplyWindowSwitcher(bool apply)
-{
-    if (m_applyWindowSwitcher == apply) {
-        return;
-    }
-    m_applyWindowSwitcher = apply;
-    emit applyWindowSwitcherChanged();
-}
-
-bool KCMLookandFeel::applyWindowSwitcher() const
-{
-    return m_applyWindowSwitcher;
-}
-
-void KCMLookandFeel::setApplyDesktopSwitcher(bool apply)
-{
-    if (m_applyDesktopSwitcher == apply) {
-        return;
-    }
-    m_applyDesktopSwitcher = apply;
-    emit applyDesktopSwitcherChanged();
-}
-
-bool KCMLookandFeel::applyDesktopSwitcher() const
-{
-    return m_applyDesktopSwitcher;
-}
-
 void KCMLookandFeel::setResetDefaultLayout(bool reset)
 {
     if (m_resetDefaultLayout == reset) {
